;
; the generic procedures
;
.DATA
_Flags_Processor_	DWORD ?


.CODE
ALIGN 16
;
; obtenir des informations sur le microprocesseur qui utilise le programme
; bit 0 = Cpuid support ?
; bit 1 = Fpu support ?
; bit 2 = Rdtsc support ?
; bit 3 = cMov support ?
; bit 4 = Mmx support ?
; bit 5 = Sse support ?
; bit 6 = Sse2 support ?
; bit 7 = Sse3 support ?
;
; Syntax :
; call ProcessorAbilities
;
; Return :
; eax et la variable _Flags_Processor_ sont dfinis avec les bits positionns
;
ProcessorAbilities PROC
		push ebx
		push ecx
		push edx
		push esi

		xor esi,esi

		pushfd
		pop eax
		mov edx,eax
		btc eax,21
		push eax
		popfd
		pushfd
		pop eax
		cmp eax,edx
		je NoCpuid
		bts esi,0

		mov eax,1
		cpuid

		bt edx,0
		jnc NoFpu
		bts esi,1
NoFpu:

		bt edx,4
		jnc NoRdtsc
		bts esi,2
NoRdtsc:

		bt edx,15
		jnc NoCmov
		bts esi,3
NoCmov:

		bt edx,23
		jnc NoMmx
		bts esi,4
NoMmx:

		bt edx,25
		jnc NoSse
		bts esi,5
NoSse:

		bt edx,26
		jnc NoSse2
		bts esi,6
NoSse2:

		bt ecx,0
		jnc NoSse3
		bts esi,7
NoSse3:

NoCpuid:
		mov _Flags_Processor_,esi
		mov eax,esi

		pop esi
		pop edx
		pop ecx
		pop ebx
	ret
ProcessorAbilities ENDP


;
; Random numbers
;
.DATA
ALIGN 16
_Random_Numbers_		DWORD ?,?,?,?


.CODE
ALIGN 16
;
; Initialiser 4 nombres aleatoires avec le nombre de millisecondes coules depuis le dmarrage de l'ordinateur.
;
; Syntax :
; call InitRandomNumbers
;
; Return:
; eax = un nombre alatoire (mais surtout, la variable _Random_Numbers_ (dword*4) est dfinie)
;
InitRandomNumbers PROC
		push ecx
		push edx

		invoke GetTickCount
		bswap eax
		test eax,eax
		jnz Label1
		mov eax,17071968
Label1:	mov _Random_Numbers_,eax
		xor eax,0CCCCCCCCh
		mov _Random_Numbers_+4,eax
		ror eax,16
		mov _Random_Numbers_+8,eax
		xor eax,0F0F0F0F0h
		mov _Random_Numbers_+12,eax

		pop edx
		pop ecx
	ret
InitRandomNumbers ENDP


ALIGN 16
;
; generer un nombre aleatoire 32-bits entre 0 et une valeur maximale (incluse)
;
; Syntax :
; mov ebx,{valeur maximale}
; call GenerateRandomNumber
;
; Return :
; eax = nombre aleatoire entre 0 et la valeur maximale (incluse)
;
GenerateRandomNumber PROC
		push edx

		mov eax,19680717
		mul _Random_Numbers_
		bswap eax
		xor eax,078787878h
		jnz Label1
		mov eax,17071968
Label1:	mov _Random_Numbers_,eax

		inc ebx
		jz Label2

		mul ebx
		mov eax,edx

Label2:	dec ebx

		pop edx
	ret
GenerateRandomNumber ENDP


;
; Files
;
.DATA
ALIGN 16
_Files_Bytes_Read_					DWORD ?
_Files_Bytes_Written_				DWORD ?
_Files_Extra_Size_					DWORD ?
_Files_Temporary_File_Pointer_		DWORD ?

ALIGN 16
_Files_File_Infos_					WIN32_FIND_DATA <?>


.CODE
ALIGN 16
;
; charger un fichier bitmap en mmoire (dans un bloc de mme taille) (ncessite comdlg32.lib)
; note : la structure OPENFILENAME concernant le fichier doit avoir t mise  jour si on utilise cette structure
;
; Syntax :
; mov ebx,{OFFSET de la chane contenant le chemin+nom du fichier (peut tre OPENFILENAME.lpstrFile)}
; mov ecx,{OFFSET d'une structure Bmp_Pointers qui recevra les pointeurs}
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Mem_Area qui recevera le fichier}
; call LoadBmpFile
;
; Return :
; eax = nombre d'octets lus (ou zro s'il y a une erreur, ou -1 si impossible d'obtenir l'info concernant la taille)
;
LoadBmpFile PROC
		push edx
		push ecx

		invoke GetCompressedFileSize,ebx,ADDR _Files_Extra_Size_
		cmp eax,INVALID_HANDLE_VALUE
		je NoFileToLoad

		mov (Mem_Area PTR [edi]).Mem_Size,eax
		mov (Mem_Area PTR [edi]).Real_Mem_Size,eax
		add eax,000000010h

		invoke HeapAlloc,(Mem_Info PTR [esi]).Instance,HEAP_NO_SERIALIZE or HEAP_ZERO_MEMORY,eax
		mov (Mem_Area PTR [edi]).Pointer_To_Free,eax
		mov ecx,eax
		and ecx,00000000Fh
		jz AlreadyOk
		and eax,0FFFFFFF0h
		add eax,000000010h
AlreadyOk:
		mov (Mem_Area PTR [edi]).Pointer,eax

		invoke CreateFile,ebx,GENERIC_READ,NULL,NULL,OPEN_EXISTING,NULL,NULL
		mov _Files_Temporary_File_Pointer_,eax
		invoke ReadFile,_Files_Temporary_File_Pointer_,(Mem_Area PTR [edi]).Pointer,(Mem_Area PTR [edi]).Mem_Size,ADDR _Files_Bytes_Read_,NULL
		invoke CloseHandle,_Files_Temporary_File_Pointer_

		pop ecx

		mov eax,(Mem_Area PTR [edi]).Pointer
		mov (Bmp_Pointers PTR [ecx]).Pointer_Bmp_Header,eax
		add eax,SIZEOF BITMAPFILEHEADER
		mov (Bmp_Pointers PTR [ecx]).Pointer_Bmp_Info,eax
		add eax,SIZEOF BITMAPINFOHEADER
		mov (Bmp_Pointers PTR [ecx]).Pointer_Bmp_Palette,eax
		mov eax,(Bmp_Pointers PTR [ecx]).Pointer_Bmp_Header
		add eax,(BITMAPFILEHEADER PTR [eax]).bfOffBits
		mov (Bmp_Pointers PTR [ecx]).Pointer_Bmp_Pixels,eax
		mov eax,_Files_Bytes_Read_

		pop edx
	ret

NoFileToLoad:

		pop ecx
		pop edx
	ret
LoadBmpFile ENDP


;
; Memory / Screens
;
.DATA
ALIGN 16


.CODE
ALIGN 16
;
; crer l'instance mmoire (ce qui doit tre fait, une fois, avant de pouvoir allouer des blocs mmoires)
; note : normalement (sauf cas exceptionnels) une seule instance suffit pour l'ensemble de votre programme
; enfin, il ne faudra pas oublier de librer l'instance en fin de programme avec SuppressMemInstance
;
; Syntax :
; mov esi,{OFFSET d'une structure Mem_Info}
; call CreateMemInstance
;
; Return :
; eax = l'adresse de l'instance
; et les variables de la structure Mem_Info sont dfinies
;
CreateMemInstance PROC
		push ecx
		push edx

		invoke HeapCreate,HEAP_NO_SERIALIZE,(Mem_Info PTR [esi]).Init_Size,(Mem_Info PTR [esi]).Max_Size
		mov (Mem_Info PTR [esi]).Instance,eax

		pop edx
		pop ecx
	ret
CreateMemInstance ENDP


ALIGN 16
;
; supprimer l'instance mmoire (ce qui doit tre fait avant de sortir de l'application, si une instance  t cre)
;
; Syntax :
; mov esi,{OFFSET d'une structure Mem_Info}
; call SuppressMemInstance
;
; Return :
; rien
;
SuppressMemInstance PROC
		push eax
		push ecx
		push edx

		invoke HeapDestroy,(Mem_Info PTR [esi]).Instance

		pop edx
		pop ecx
		pop eax
	ret
SuppressMemInstance ENDP


ALIGN 16
;
; allouer un bloc mmoire  un pointeur (ce bloc est align sur 16 octets)
; note : la structure Mem_Info doit, videment, avoir t au pralable cre et dfinie
; enfin, comme ici on alloue un bloc mmoire, il faudra le librer en fin de programme avec LibererBlocMemoire
;
; Syntax :
; mov eax,{taille du bloc mmoire que l'on va crer (en octets)}
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Mem_Area}
; call AllocMemArea
;
; Return :
; eax = adresse du bloc mmoire align
; et les variables de la structure Mem_Area sont dfinies
;
AllocMemArea PROC
		push ecx
		push edx

		mov (Mem_Area PTR [edi]).Mem_Size,eax
		mov (Mem_Area PTR [edi]).Real_Mem_Size,eax
		add eax,000000010h
		invoke HeapAlloc,(Mem_Info PTR [esi]).Instance,HEAP_NO_SERIALIZE or HEAP_ZERO_MEMORY,eax
		mov (Mem_Area PTR [edi]).Pointer_To_Free,eax
		mov ecx,eax
		and ecx,00000000Fh
		jz AlreadyOk
		and eax,0FFFFFFF0h
		add eax,000000010h
AlreadyOk:
		mov (Mem_Area PTR [edi]).Pointer,eax

		pop edx
		pop ecx
	ret
AllocMemArea ENDP


ALIGN 16
;
; librer un bloc mmoire
; note : si il n'y a pas d'autres bloc mmoire, ne pas oublier de librer l'instance mmoire en fin de programme
;
; Syntax :
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Mem_Area}
; call SuppressMemArea
;
; Return :
; rien
;
SuppressMemArea PROC
		push eax
		push ecx
		push edx

		invoke HeapFree,(Mem_Info PTR [esi]).Instance,HEAP_NO_SERIALIZE,(Mem_Area PTR [edi]).Pointer_To_Free

		pop edx
		pop ecx
		pop eax
	ret
SuppressMemArea ENDP


ALIGN 16
;
; allouer un cran (bloc mmoire)  un pointeur (ce bloc est align sur 16 octets)
; note : la structure Mem_Info doit, videment, avoir t au pralable cre et dfinie
; enfin, comme ici on alloue un bloc mmoire, il faudra le librer en fin de programme avec SuppressScreenArea
;
; Syntax :
; mov eax,{largeur de l'cran (en pixels)}
; mov ebx,{hauteur de l'cran (en pixels)}
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Screen_Area}
; call CreateScreenArea
;
; Return :
; eax = adresse du bloc mmoire align (ce sera le tampon de l'cran)
; et les variables de la structure Screen_Area sont dfinies
;
CreateScreenArea PROC
		push ecx
		push edx

		mov (Screen_Area PTR [edi]).Screen_Width,eax
		mov (Screen_Area PTR [edi]).Screen_Height,ebx
		shl eax,2
		mov (Screen_Area PTR [edi]).Screen_Width_x4,eax
		mul ebx
		mov (Screen_Area PTR [edi]).Screen_Size,eax
		mov (Screen_Area PTR [edi]).Real_Screen_Size,eax
		add eax,000000010h
		invoke HeapAlloc,(Mem_Info PTR [esi]).Instance,HEAP_NO_SERIALIZE or HEAP_ZERO_MEMORY,eax
		mov (Screen_Area PTR [edi]).Pointer_To_Free,eax
		mov ecx,eax
		and ecx,00000000Fh
		jz AlreadyOk
		and eax,0FFFFFFF0h
		add eax,000000010h
AlreadyOk:
		mov (Screen_Area PTR [edi]).Pointer,eax
		mov (Screen_Area PTR [edi]).Pointer_Dbuffer,eax
		mov (Screen_Area PTR [edi]).Pointer_Zbuffer,eax
		mov (Screen_Area PTR [edi]).Pointer_Face_Limits,eax

		pop edx
		pop ecx
	ret
CreateScreenArea ENDP


ALIGN 16
;
; allouer un cran (bloc mmoire)  un pointeur (ce bloc est align sur 16 octets), et alloue galement un autre cran  un deuxime
; pointeur (Dbuffer, qui sert pour viter l'effet flicker)
; note : la structure Mem_Info doit, videment, avoir t au pralable cre et dfinie
; la taille cran doit tre multiple de 4 pixels (16 octets, sinon il y aura crash si utilisation d'instructions sse...)
; note 2 : au lieu d'utiliser le pointeur cr pour Dbuffer, ajouter ou enlever la taille du bloc au pointeur
; normal sera beaucoup plus rapide que d'effectuer des tas de tests pour savoir sur quel cran on travail...
; enfin, comme ici on alloue un bloc mmoire, il faudra le librer en fin de programme avec SuppressScreenArea
;
; Syntax :
; mov eax,{largeur de l'cran (en pixels)}
; mov ebx,{hauteur de l'cran (en pixels)}
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Screen_Area}
; call CreateDBufferScreenArea
;
; Return :
; eax = adresse du bloc mmoire align (ce sera le tampon de l'cran)
; et les variables de la structure Screen_Area sont dfinies
;
CreateDBufferScreenArea PROC
		push ecx
		push edx

		mov (Screen_Area PTR [edi]).Screen_Width,eax
		mov (Screen_Area PTR [edi]).Screen_Height,ebx
		shl eax,2
		mov (Screen_Area PTR [edi]).Screen_Width_x4,eax
		mul ebx
		mov (Screen_Area PTR [edi]).Screen_Size,eax
		add eax,eax
		mov (Screen_Area PTR [edi]).Real_Screen_Size,eax
		add eax,000000010h
		invoke HeapAlloc,(Mem_Info PTR [esi]).Instance,HEAP_NO_SERIALIZE or HEAP_ZERO_MEMORY,eax
		mov (Screen_Area PTR [edi]).Pointer_To_Free,eax
		mov ecx,eax
		and ecx,00000000Fh
		jz AlreadyOk
		and eax,0FFFFFFF0h
		add eax,000000010h
AlreadyOk:
		mov (Screen_Area PTR [edi]).Pointer,eax
		mov (Screen_Area PTR [edi]).Pointer_Zbuffer,eax
		mov edx,eax
		add edx,(Screen_Area PTR [edi]).Screen_Size
		mov (Screen_Area PTR [edi]).Pointer_Dbuffer,edx
		mov (Screen_Area PTR [edi]).Pointer_Face_Limits,edx

		pop edx
		pop ecx
	ret
CreateDBufferScreenArea ENDP


ALIGN 16
;
; effacer un bloc memoire (en le remplissant de 0)
; note : MM0 est modifi
;
; Syntax :
; mov edi,{OFFSET d'une structure Screen_Area}
; call CleanDBuffer_Mmx
;
; Return :
; eax = taille du bloc cran en octets
;
CleanDBuffer_Mmx PROC
		push ecx
		push edi

		pxor MM0,MM0
		mov eax,(Screen_Area PTR [edi]).Screen_Size
		mov edi,(Screen_Area PTR [edi]).Pointer_Dbuffer
		mov ecx,eax
		and ecx,11111111111111111111111111000000b
		jz Label1

Label0:	movq QWORD PTR[edi],MM0
		movq QWORD PTR[edi+8],MM0
		movq QWORD PTR[edi+16],MM0
		movq QWORD PTR[edi+24],MM0
		movq QWORD PTR[edi+32],MM0
		movq QWORD PTR[edi+40],MM0
		movq QWORD PTR[edi+48],MM0
		movq QWORD PTR[edi+56],MM0
		add edi,QWORD*8
		sub ecx,QWORD*8
		jnz Label0

Label1:	mov ecx,eax
		and ecx,00000000000000000000000000110000b
		jz Label3
		add edi,ecx
		neg ecx
Label2:	movq QWORD PTR[edi+ecx],MM0
		movq QWORD PTR[edi+ecx+8],MM0
		add ecx,QWORD*2
		jnz Label2

Label3:	mov ecx,eax
		and ecx,00000000000000000000000000001100b
		jz Label5
		add edi,ecx
		neg ecx
Label4:	mov DWORD PTR [edi+ecx],0
		add ecx,DWORD
		jnz Label4

Label5:
		pop edi
		pop ecx
	ret
CleanDBuffer_Mmx ENDP


ALIGN 16
;
; copier un bloc memoire vers un autre bloc memoire
; note : il faut aussi s'assurer que les blocs mmoire ne se chevauchent pas, sinon il faut utiliser
; MoveMemArea_Mmx... tous les registres MMx sont utiliss
;
; Syntax :
; mov eax,{taille  copier en octets}
; mov esi,{OFFSET du bloc source}
; mov edi,{OFFSET du bloc de destination}
; call CopyMemArea_Mmx
;
; Return :
; rien
;
CopyMemArea_Mmx PROC
		push ecx
		push edx
		push esi
		push edi

		sub edi,esi
		mov ecx,eax
		and ecx,11111111111111111111111111000000b
		jz Label1

Label0:	movq MM0,QWORD PTR[esi]
		movq MM1,QWORD PTR[esi+8]
		movq MM2,QWORD PTR[esi+16]
		movq MM3,QWORD PTR[esi+24]
		movq MM4,QWORD PTR[esi+32]
		movq MM5,QWORD PTR[esi+40]
		movq MM6,QWORD PTR[esi+48]
		movq MM7,QWORD PTR[esi+56]
		movq QWORD PTR[esi+edi],MM0
		movq QWORD PTR[esi+edi+8],MM1
		movq QWORD PTR[esi+edi+16],MM2
		movq QWORD PTR[esi+edi+24],MM3
		movq QWORD PTR[esi+edi+32],MM4
		movq QWORD PTR[esi+edi+40],MM5
		movq QWORD PTR[esi+edi+48],MM6
		movq QWORD PTR[esi+edi+56],MM7
		add esi,QWORD*8
		sub ecx,QWORD*8
		jnz Label0

Label1:	mov ecx,eax
		and ecx,00000000000000000000000000110000b
		jz Label3
Label2:	movq MM0,QWORD PTR[esi]
		movq MM1,QWORD PTR[esi+8]
		movq QWORD PTR[esi+edi],MM0
		movq QWORD PTR[esi+edi+8],MM1
		add esi,QWORD*2
		sub ecx,QWORD*2
		jnz Label2

Label3:	mov ecx,eax
		and ecx,00000000000000000000000000001100b
		jz Label5
Label4:	mov edx,DWORD PTR [esi]
		mov DWORD PTR [esi+edi],edx
		add esi,DWORD
		sub ecx,DWORD
		jnz Label4

Label5:	mov ecx,eax
		and ecx,00000000000000000000000000000011b
		jz Label7
Label6:	mov dl,BYTE PTR [esi]
		mov BYTE PTR [esi+edi],dl
		add esi,BYTE
		sub ecx,BYTE
		jnz Label6
Label7:
		pop edi
		pop esi
		pop edx
		pop ecx
	ret
CopyMemArea_Mmx ENDP


ALIGN 16
;
; copier un bloc memoire vers un autre bloc memoire (c'est une version de CopyMemArea_Mmx qui tiens compte des possibles chevauchements)
; note : tous les registres MMx sont utiliss
;
; Syntax :
; mov eax,{taille  copier en octets}
; mov esi,{OFFSET du bloc source}
; mov edi,{OFFSET du bloc de destination}
; call MoveMemArea_Mmx
;
; Return :
; rien
;
MoveMemArea_Mmx PROC
		push ecx
		push edx

		cmp esi,edi
		jae Label00
		mov ecx,edi
		sub ecx,esi
		cmp eax,ecx
		ja Label09

Label00:	mov ecx,eax
		and ecx,11111111111111111111111111000000b
		jz Label02

Label01:	movq MM0,QWORD PTR[esi]
		movq MM1,QWORD PTR[esi+8]
		movq MM2,QWORD PTR[esi+16]
		movq MM3,QWORD PTR[esi+24]
		movq MM4,QWORD PTR[esi+32]
		movq MM5,QWORD PTR[esi+40]
		movq MM6,QWORD PTR[esi+48]
		movq MM7,QWORD PTR[esi+56]
		movq QWORD PTR[edi],MM0
		movq QWORD PTR[edi+8],MM1
		movq QWORD PTR[edi+16],MM2
		movq QWORD PTR[edi+24],MM3
		movq QWORD PTR[edi+32],MM4
		movq QWORD PTR[edi+40],MM5
		movq QWORD PTR[edi+48],MM6
		movq QWORD PTR[edi+56],MM7
		add esi,QWORD*8
		add edi,QWORD*8
		sub ecx,QWORD*8
		jnz Label01

Label02:	mov ecx,eax
		and ecx,00000000000000000000000000110000b
		jz Label04
		add esi,ecx
		add edi,ecx
		neg ecx
Label03:	movq MM0,QWORD PTR[esi+ecx]
		movq MM1,QWORD PTR[esi+ecx+8]
		movq QWORD PTR[edi+ecx],MM0
		movq QWORD PTR[edi+ecx+8],MM1
		add ecx,QWORD*2
		jnz Label03

Label04:	mov ecx,eax
		and ecx,00000000000000000000000000001100b
		jz Label06
		add esi,ecx
		add edi,ecx
		neg ecx
Label05:	mov edx,DWORD PTR [esi+ecx]
		mov DWORD PTR [edi+ecx],edx
		add ecx,DWORD
		jnz Label05

Label06:	mov ecx,eax
		and ecx,00000000000000000000000000000011b
		jz Label08
		add esi,ecx
		add edi,ecx
		neg ecx
Label07:	mov dl,BYTE PTR [esi+ecx]
		mov BYTE PTR [edi+ecx],dl
		inc ecx
		jnz Label07
Label08:	sub esi,eax
		sub edi,eax

		pop edx
		pop ecx
	ret

ALIGN 16

Label09:	add esi,eax
		add edi,eax

		mov ecx,eax
		and ecx,00000000000000000000000000000011b
		jz Label11
		sub esi,ecx
		sub edi,ecx
Label10:	dec ecx
		mov dl,BYTE PTR [esi+ecx]
		mov BYTE PTR [edi+ecx],dl
		jnz Label10

Label11:	mov ecx,eax
		and ecx,00000000000000000000000000001100b
		jz Label13
		sub esi,ecx
		sub edi,ecx
Label12:	sub ecx,4
		mov edx,DWORD PTR [esi+ecx]
		mov DWORD PTR [edi+ecx],edx
		jnz Label12

Label13:	mov ecx,eax
		and ecx,00000000000000000000000000110000b
		jz Label15
		sub esi,ecx
		sub edi,ecx
Label14:	sub ecx,16
		movq MM0,QWORD PTR[esi+ecx+8]
		movq MM1,QWORD PTR[esi+ecx]
		movq QWORD PTR[edi+ecx+8],MM0
		movq QWORD PTR[edi+ecx],MM1
		jnz Label14

Label15:	mov ecx,eax
		and ecx,11111111111111111111111111000000b
		jz Label17
		sub esi,ecx
		sub edi,ecx

Label16:	sub ecx,64
		movq MM0,QWORD PTR[esi+ecx+56]
		movq MM1,QWORD PTR[esi+ecx+48]
		movq MM2,QWORD PTR[esi+ecx+40]
		movq MM3,QWORD PTR[esi+ecx+32]
		movq MM4,QWORD PTR[esi+ecx+24]
		movq MM5,QWORD PTR[esi+ecx+16]
		movq MM6,QWORD PTR[esi+ecx+8]
		movq MM7,QWORD PTR[esi+ecx]
		movq QWORD PTR[edi+ecx+56],MM0
		movq QWORD PTR[edi+ecx+48],MM1
		movq QWORD PTR[edi+ecx+40],MM2
		movq QWORD PTR[edi+ecx+32],MM3
		movq QWORD PTR[edi+ecx+24],MM4
		movq QWORD PTR[edi+ecx+16],MM5
		movq QWORD PTR[edi+ecx+8],MM6
		movq QWORD PTR[edi+ecx],MM7
		jnz Label16
Label17:
		pop edx
		pop ecx
	ret
MoveMemArea_Mmx ENDP


ALIGN 16
;
; dcaler (de x octets) un bloc mmoire
; note : la structure Mem_Info doit, videment, avoir t au pralable cre et dfinie
; et le dcalage se fait sur la gauche, c'est un parti pris que j'assume... tous les registres MMx sont utiliss
;
; Syntax :
; mov eax,{pas (nombre d'octets) de dcalage}
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Mem_Area, qui concerne le bloc mmoire  dcaler}
; call DecayMemArea_Mmx
;
; Return :
; rien
;
DecayMemArea_Mmx PROC
		pushad

		mov ebx,esi
		mov ebp,edi

		mov edi,OFFSET _Mem_Temporary_Area_
		call AllocMemArea

		mov eax,(Mem_Area PTR [edi]).Mem_Size
		mov esi,(Mem_Area PTR [ebp]).Pointer
		mov edi,(Mem_Area PTR [edi]).Pointer
		call CopyMemArea_Mmx

		add esi,eax
		mov edi,(Mem_Area PTR [ebp]).Pointer
		mov ecx,eax
		mov eax,(Mem_Area PTR [ebp]).Mem_Size
		sub eax,ecx
		call MoveMemArea_Mmx

		add edi,eax
		mov esi,_Mem_Temporary_Area_.Pointer
		mov eax,_Mem_Temporary_Area_.Mem_Size
		call CopyMemArea_Mmx

		mov esi,ebx
		mov edi,OFFSET _Mem_Temporary_Area_
		call SuppressMemArea

		popad
	ret
DecayMemArea_Mmx ENDP


ALIGN 16
;
; mettre  jour l'cran sur lequel on va travailler (cette routine n'est utile que lorsqu'il y a un dbuffer. et
; ne devrait pas tre employe s'il n'y en a pas... mme si ca ne causera pas de crash). cette mise  jour doit
; tre positionne au dbut de la boucle principale de votre programme (ou tout au moins avant d'crire pour la
; premire fois dans l'cran), ou alors en toute fin de boucle (aprs l'utilisation de SetDIBitsToDevice)
;
; Syntax :
; mov edi,{OFFSET d'une structure Screen_Area}
; call UpdateDBuffer
;
; Return :
; rien
;
UpdateDBuffer PROC
		push (Screen_Area PTR [edi]).Pointer
		push (Screen_Area PTR [edi]).Pointer_Dbuffer
		pop (Screen_Area PTR [edi]).Pointer
		pop (Screen_Area PTR [edi]).Pointer_Dbuffer
	ret
UpdateDBuffer ENDP


ALIGN 16
;
; librer un cran (bloc mmoire)
; note : si il n'y a pas d'autres bloc mmoire, ne pas oublier de supprimer l'intance mmoire
;
; Syntax :
; mov esi,{OFFSET d'une structure Mem_Info}
; mov edi,{OFFSET d'une structure Screen_Area}
; call SuppressScreenArea
;
; Return :
; rien
;
SuppressScreenArea PROC
		push eax
		push ecx
		push edx

		invoke HeapFree,(Mem_Info PTR [esi]).Instance,HEAP_NO_SERIALIZE,(Screen_Area PTR [edi]).Pointer_To_Free

		pop edx
		pop ecx
		pop eax
	ret
SuppressScreenArea ENDP


;
; Graphics
;
.DATA
ALIGN 16
_Mem_Simd_Mask_RVB_unpck_1_		DWORD 000010001h,000000001h,000010001h,000000001h
_Mem_Simd_Mask_RVB_unpck_256_		DWORD 001000100h,000000100h,001000100h,000000100h

_Mem_Simd_Mask_Reds_			DWORD 000FF0000h,000FF0000h,000FF0000h,000FF0000h
_Mem_Simd_Mask_Blues_			DWORD 0000000FFh,0000000FFh,0000000FFh,0000000FFh

_Mem_Simd_Mask_w255_			DWORD 000FF00FFh,000FF00FFh,000FF00FFh,000FF00FFh

_Sprite_Width_Loop_				DWORD ?
_Sprite_Height_Loop_			DWORD ?
_Sprite_Line_Complement_			DWORD ?

ALIGN 16
_Lines_TestValue_				DWORD ?
_Lines_Counter_				DWORD ?

ALIGN 16
_Mem_Temporary_Area_			Mem_Area <?>


.CODE
ALIGN 16
;
; elargir un cran source, dans un cran destination (ils doivent tre diffrents...)
; note : ici, ne fonctionne qu'en largissant
;
; Syntax :
; mov esi,{OFFSET d'une structure Screen_Area (le tampon de l'cran originel)}
; mov edi,{OFFSET d'une structure Screen_Area (le tampon de l'cran final)}
; call EnlargeScreenArea
;
; Return :
; rien
;
EnlargeScreenArea PROC
		pushad

		mov eax,(Screen_Area PTR [esi]).Screen_Height
		mov ecx,(Screen_Area PTR [esi]).Screen_Width_x4
		mov esi,(Screen_Area PTR [esi]).Pointer
		mov ebx,(Screen_Area PTR [edi]).Screen_Height
		mov edx,(Screen_Area PTR [edi]).Screen_Width_x4
		mov edi,(Screen_Area PTR [edi]).Pointer
		mov ebp,ebx
		neg ebp
		mov _Lines_Counter_,ebx
		mov _Lines_TestValue_,ebp

Label01:	push eax
		push ebx
		push esi

		mov ebx,edx
		mov ebp,edx
		add edi,edx
		neg ebx
		neg ebp

Label02:	mov eax,DWORD PTR [esi]
		add ebx,ecx
		js Label03
		add esi,DWORD
		sub ebx,edx

Label03:	mov DWORD PTR [edi+ebp],eax
		add ebp,DWORD
		jnz Label02

		pop esi
		pop ebx
		pop eax

		add _Lines_TestValue_,eax
		js Label04
		add esi,ecx
		sub _Lines_TestValue_,ebx

Label04:	dec _Lines_Counter_
		jnz Label01

		popad
	ret
EnlargeScreenArea ENDP


ALIGN 16
;
; initialiser un sprite (on va remplir une structure Sprite_Info relatif au sprite)
;
; Syntax :
; mov edx,{OFFSET des donnes du sprite}
; mov esi,{OFFSET de la structure Sprite_Info}
; mov edi,{OFFSET de la structure Screen_Area de l'cran de destination}
; call InitSprite
;
; Return :
; rien
;
InitSprite PROc
		push eax
		push edx

		mov eax,SIZEOF Sprite_Info
		mov (Sprite_Info PTR [esi]).Structure_Lenght,eax
		mov eax,[edx+12]
		mov (Sprite_Info PTR [esi]).Transparent_Color,eax
		mov eax,[edx]
		mov (Sprite_Info PTR [esi]).Sprite_Width,eax
		mov edx,[edx+4]
		mov (Sprite_Info PTR [esi]).Sprite_Height,edx
		mul edx
		mov (Sprite_Info PTR [esi]).Sprite_Size,eax
		shl eax,2
		mov (Sprite_Info PTR [esi]).Sprite_Size_x4,eax
		mov eax,(Sprite_Info PTR [esi]).Sprite_Width
		shl eax,2
		mov (Sprite_Info PTR [esi]).Sprite_Width_x4,eax

		mov eax,(Screen_Area PTR [edi]).Screen_Width_x4
		mov (Sprite_Info PTR [esi]).Screen_Width_x4,eax
		mul (Sprite_Info PTR [esi]).Sprite_Height
		mov (Sprite_Info PTR [esi]).Virtual_Height,eax
		mov eax,(Screen_Area PTR [edi]).Screen_Width_x4
		sub eax,(Sprite_Info PTR [esi]).Sprite_Width_x4
		mov (Sprite_Info PTR [esi]).Screen_Width_Complement,eax

		mov (Sprite_Info PTR [esi]).Transparency_Level,000FFFFFFh

		pop edx
		pop eax
	ret
InitSprite ENDP


ALIGN 16
;
; placer un alpha sprite centr  un emplacement X,Y de l'cran
;
; Syntax :
; mov eax,{position X du sprite  afficher}
; mov ebx,{position Y du sprite  afficher}
; mov edx,{couleur du sprite}
; mov esi,{OFFSET de la structure Sprite_Info}
; mov edi,{OFFSET de la structure Screen_Area de l'cran de destination}
; call DisplayCenteredAlphaSprite_Mmx
;
; Return :
; rien, (attention, tous les registre MMx sont modifis)
;
DisplayCenteredAlphaSprite_Mmx PROC
		movd MM5,edx
		movq MM6,QWORD PTR _Mem_Simd_Mask_w255_
		pxor MM7,MM7

		pushad

		mov ebp,edi
		mov ecx,(Sprite_Info PTR [esi]).Sprite_Width
		mov edx,(Sprite_Info PTR [esi]).Sprite_Height
		shr ecx,1
		shr edx,1
		sub eax,ecx
		sub ebx,edx
		mov ecx,eax
		mov edx,ebx
		add ecx,(Sprite_Info PTR [esi]).Sprite_Width
		add edx,(Sprite_Info PTR [esi]).Sprite_Height

		test eax,eax
		js Label001
		cmp eax,(Screen_Area PTR [ebp]).Screen_Width
		jge Label002
		cmp ecx,(Screen_Area PTR [ebp]).Screen_Width
		jae Label009
		test ebx,ebx
		js Label003
		cmp ebx,(Screen_Area PTR [ebp]).Screen_Height
		jge Label004
		cmp edx,(Screen_Area PTR [ebp]).Screen_Height
		jae Label009
		jmp Label005

Label001:	test ecx,ecx
		jns Label009
		popad
	ret
Label002:	cmp ecx,(Screen_Area PTR [ebp]).Screen_Width
		jl Label009
		popad
	ret
Label003:	test edx,edx
		jns Label009
		popad
	ret
Label004:	cmp edx,(Screen_Area PTR [ebp]).Screen_Height
		jl Label009
		popad
	ret

Label005:	shl eax,2
		mov (Sprite_Info PTR [esi]).X_Position,eax
		mov edi,(Screen_Area PTR [ebp]).Pointer

		mov eax,(Screen_Area PTR [ebp]).Screen_Width_x4
		imul ebx
		mov (Sprite_Info PTR [esi]).Y_Position,eax
		add edi,eax

		mov ecx,esi
		add esi,(Sprite_Info PTR [ecx]).Structure_Lenght
		add edi,(Sprite_Info PTR [ecx]).X_Position
		mov eax,(Sprite_Info PTR [ecx]).Sprite_Height
		mov _Sprite_Height_Loop_,eax
		mov eax,(Sprite_Info PTR [ecx]).Transparent_Color
		mov ebx,(Screen_Area PTR [ebp]).Screen_Width_x4
		sub ebx,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		mov _Sprite_Line_Complement_,ebx

Label006:	mov ebx,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		add esi,ebx
		add edi,ebx
		neg ebx

Label007:	mov edx,DWORD PTR [esi+ebx]
		cmp edx,eax
		je Label008

		movd MM0,DWORD PTR [edi+ebx]
		movq MM1,MM5
		movd MM2,edx
		movd MM3,edx
		movd MM4,edx
		psrlq MM2,8
		psrlq MM3,16
		psrlq MM4,24
		por MM2,MM3
		por MM2,MM4
		punpcklbw MM2,MM7
		movq MM3,MM2
		pxor MM3,MM6
		punpcklbw MM0,MM7
		punpcklbw MM1,MM7
		pmullw MM1,MM2
		pmullw MM0,MM3
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM0
		movd DWORD PTR [edi+ebx],MM0

Label008:	add ebx,4
		jnz Label007

		add edi,_Sprite_Line_Complement_

		dec _Sprite_Height_Loop_
		jnz Label006

		popad
	ret

Label009:	shl eax,2
		mov (Sprite_Info PTR [esi]).X_Position,eax
		mov edi,(Screen_Area PTR [ebp]).Pointer

		mov eax,(Screen_Area PTR [ebp]).Screen_Width_x4
		imul ebx
		mov (Sprite_Info PTR [esi]).Y_Position,eax
		add edi,eax

		mov ecx,esi
		add esi,(Sprite_Info PTR [ecx]).Structure_Lenght
		mov eax,(Sprite_Info PTR [ecx]).Sprite_Size_x4
		add esi,eax
		neg eax
		mov ebx,(Sprite_Info PTR [ecx]).X_Position

Label010:	mov edx,edi
		sub edx,(Screen_Area PTR [ebp]).Pointer
		cmp edx,(Screen_Area PTR [ebp]).Screen_Size
		jae Label013

		mov edx,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		mov _Sprite_Width_Loop_,edx

Label011:	cmp ebx,(Screen_Area PTR [ebp]).Screen_Width_x4
		jae Label012
		mov edx,DWORD PTR [esi+eax]
		cmp edx,(Sprite_Info PTR [ecx]).Transparent_Color
		je Label012

		movd MM0,DWORD PTR [edi+ebx]
		movq MM1,MM5
		movd MM2,edx
		movd MM3,edx
		movd MM4,edx
		psrlq MM2,8
		psrlq MM3,16
		psrlq MM4,24
		por MM2,MM3
		por MM2,MM4
		punpcklbw MM2,MM7
		movq MM3,MM2
		pxor MM3,MM6
		punpcklbw MM0,MM7
		punpcklbw MM1,MM7
		pmullw MM1,MM2
		pmullw MM0,MM3
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM0
		movd DWORD PTR [edi+ebx],MM0
Label012:	add eax,4
		add ebx,4

		sub _Sprite_Width_Loop_,4
		jnz Label011

		sub ebx,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		add edi,(Screen_Area PTR [ebp]).Screen_Width_x4

		test eax,eax
		jnz Label010

		popad
	ret

Label013:	add edi,(Screen_Area PTR [ebp]).Screen_Width_x4

		add eax,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		jnz Label010

		popad
	ret
DisplayCenteredAlphaSprite_Mmx ENDP


ALIGN 16
;
; placer un sprite centr  un emplacement X,Y de l'cran
;
; Syntax :
; mov eax,{position X du sprite  afficher}
; mov ebx,{position Y du sprite  afficher}
; mov esi,{OFFSET de la structure Sprite_Info}
; mov edi,{OFFSET de la structure Screen_Area de l'cran de destination}
; call DisplayCenteredSprite_Mmx
;
; Return :
; rien, (attention, tous les registre MMx sont modifis)
;
DisplayCenteredSprite_Mmx PROC

		pushad

		mov ebp,edi
		mov ecx,(Sprite_Info PTR [esi]).Sprite_Width
		mov edx,(Sprite_Info PTR [esi]).Sprite_Height
		shr ecx,1
		shr edx,1
		sub eax,ecx
		sub ebx,edx
		mov ecx,eax
		mov edx,ebx
		add ecx,(Sprite_Info PTR [esi]).Sprite_Width
		add edx,(Sprite_Info PTR [esi]).Sprite_Height

		test eax,eax
		js Label11
		cmp eax,(Screen_Area PTR [ebp]).Screen_Width
		jge Label12
		cmp ecx,(Screen_Area PTR [ebp]).Screen_Width
		jae Label21
		test ebx,ebx
		js Label13
		cmp ebx,(Screen_Area PTR [ebp]).Screen_Height
		jge Label14
		cmp edx,(Screen_Area PTR [ebp]).Screen_Height
		jae Label21

Label01:	movq MM5,QWORD PTR _Mem_Simd_Mask_RVB_unpck_1_
		movq MM6,QWORD PTR _Mem_Simd_Mask_RVB_unpck_256_
		pxor MM7,MM7
		shl eax,2
		mov (Sprite_Info PTR [esi]).X_Position,eax
		mov edi,(Screen_Area PTR [ebp]).Pointer

		mov eax,(Screen_Area PTR [ebp]).Screen_Width_x4
		imul ebx
		add edi,eax
		mov (Sprite_Info PTR [esi]).Y_Position,eax

		mov ecx,esi
		mov eax,(Sprite_Info PTR [esi]).Sprite_Height
		add esi,(Sprite_Info PTR [esi]).Structure_Lenght
		add edi,(Sprite_Info PTR [ecx]).X_Position
		mov _Sprite_Height_Loop_,eax
		mov eax,(Sprite_Info PTR [ecx]).Transparent_Color

		mov ebp,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		mov ecx,(Sprite_Info PTR [ecx]).Screen_Width_Complement
Label02:	mov ebx,ebp
		add esi,ebp
		add edi,ebp
		neg ebx

Label03:	mov edx,DWORD PTR [esi+ebx]
		cmp edx,eax
		je Label04

		movd MM0,edx
		movd MM1,DWORD PTR [edi+ebx]
		punpcklbw MM0,MM7
		punpcklbw MM1,MM7
		movq MM2,MM0
		pshufw MM0,MM0,0FFh
		movq MM3,MM6
		psubw MM3,MM0
		paddw MM0,MM5
		pmullw MM1,MM3
		pmullw MM0,MM2
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM7
		movd DWORD PTR [edi+ebx],MM0

Label04:	add ebx,DWORD
		jnz Label03

		add edi,ecx

		dec _Sprite_Height_Loop_
		jnz Label02

		popad
	ret

Label11:	test ecx,ecx
		jns Label21
		popad
	ret
Label12:	cmp ecx,(Screen_Area PTR [ebp]).Screen_Width
		jl Label21
		popad
	ret
Label13:	test edx,edx
		jns Label21
		popad
	ret
Label14:	cmp edx,(Screen_Area PTR [ebp]).Screen_Height
		jl Label21
		popad
	ret

Label21:	movq MM5,QWORD PTR _Mem_Simd_Mask_RVB_unpck_1_
		movq MM6,QWORD PTR _Mem_Simd_Mask_RVB_unpck_256_
		pxor MM7,MM7
		shl eax,2
		mov (Sprite_Info PTR [esi]).X_Position,eax
		mov edi,(Screen_Area PTR [ebp]).Pointer

		mov eax,(Screen_Area PTR [ebp]).Screen_Width_x4
		imul ebx
		mov (Sprite_Info PTR [esi]).Y_Position,eax
		add edi,eax

		mov ecx,esi
		mov ebx,(Sprite_Info PTR [esi]).X_Position
		mov eax,(Sprite_Info PTR [esi]).Sprite_Size_x4
		add esi,(Sprite_Info PTR [esi]).Structure_Lenght
		add esi,eax
		neg eax

Label22:	mov edx,edi
		sub edx,(Screen_Area PTR [ebp]).Pointer
		cmp edx,(Screen_Area PTR [ebp]).Screen_Size
		jae Label31

		mov edx,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		mov _Sprite_Width_Loop_,edx

Label23:	cmp ebx,(Sprite_Info PTR [ecx]).Screen_Width_x4
		jae Label24
		mov edx,DWORD PTR [esi+eax]
		cmp edx,(Sprite_Info PTR [ecx]).Transparent_Color
		je Label24

		movd MM0,edx
		movd MM1,DWORD PTR [edi+ebx]
		punpcklbw MM0,MM7
		punpcklbw MM1,MM7
		movq MM2,MM0
		pshufw MM0,MM0,0FFh
		movq MM3,MM6
		psubw MM3,MM0
		paddw MM0,MM5
		pmullw MM1,MM3
		pmullw MM0,MM2
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM7
		movd DWORD PTR [edi+ebx],MM0

Label24:	add eax,DWORD
		add ebx,DWORD
		sub _Sprite_Width_Loop_,DWORD
		jnz Label23

		sub ebx,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		add edi,(Sprite_Info PTR [ecx]).Screen_Width_x4

		test eax,eax
		jnz Label22

		popad
	ret

Label31:	add edi,(Sprite_Info PTR [ecx]).Screen_Width_x4

		add eax,(Sprite_Info PTR [ecx]).Sprite_Width_x4
		jnz Label22

		popad
	ret
DisplayCenteredSprite_Mmx ENDP


ALIGN 16
;
; fusionner l'cran en cours avec l'cran prcdant (necessite le double buffet), par traitement alpha, pour la transparence des pixels
;
; Syntax :
; mov edi,{OFFSET de la structure Screen_Area de l'cran}
; call CombinePreviousScreen_Mmx
;
; Return :
; rien
;
CombinePreviousScreen_Mmx PROC
		push ecx

		mov ecx,(Screen_Area PTR [edi]).Screen_Size
		test ecx,ecx
		jz NoWork
		push edx
		push esi
		push edi

		mov edx,0007F7F7Fh
		mov esi,(Screen_Area PTR [edi]).Pointer_Dbuffer
		mov edi,(Screen_Area PTR [edi]).Pointer
		add esi,ecx
		add edi,ecx
		neg ecx

		pxor MM7,MM7
		movd MM2,edx
		punpcklbw MM2,MM7
		movq MM3,QWORD PTR _Mem_Simd_Mask_RVB_unpck_256_
		psubw MM3,MM2
		paddw MM2,QWORD PTR _Mem_Simd_Mask_RVB_unpck_1_
Pixels:	movd MM1,DWORD PTR [esi+ecx]
		movd MM0,DWORD PTR [edi+ecx]
		punpcklbw MM1,MM7
		punpcklbw MM0,MM7
		pmullw MM1,MM2
		pmullw MM0,MM3
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM7
		movd DWORD PTR [edi+ecx],MM0
		add ecx,DWORD
		jnz Pixels

		pop edi
		pop esi
		pop edx
NoWork:
		pop ecx
	ret
CombinePreviousScreen_Mmx ENDP


ALIGN 16
;
; crer un effet blur statique (le traitement doit se faire par dword sinon le rsultat est moins probant)
;
; Syntax :
; mov eax,{Largeur en pixels  traiter (doit tre infrieur ou gal  la largeur de l'cran)}
; mov ebx,{Hauteur en pixels  traiter (doit tre infrieur ou gal  la hauteur de l'cran)}
; mov ecx,{position X (en pixel, de 0  Largeur) de dbut de l'effet fondu}
; mov edx,{position Y (en pixel, de 0  hauteur) de dbut de l'effet fondu}
; mov edi,{OFFSET d'une structure Screen_Area (adresse du tampon cran)}
; call ScreenFilter_Blur_Mmx
;
; Return :
; rien
;
ScreenFilter_Blur_Mmx PROC
		pushad

		shl eax,2
		shl ecx,2
		mov esi,eax
		mov eax,edx
		mul (Screen_Area PTR [edi]).Screen_Width_x4
		add eax,ecx
		mov ecx,(Screen_Area PTR [edi]).Screen_Width_x4
		mov edi,(Screen_Area PTR [edi]).Pointer
		add edi,eax
		mov eax,esi
		sub ecx,eax
		mov ebp,ecx
		neg ebx

		pxor MM7,MM7

Label1:	mov ecx,eax
		add edi,eax
		mov edx,edi
		sub edx,eax
		sub edx,ebp
		mov esi,edi
		add esi,eax
		add esi,ebp
		neg ecx
		movd MM1,DWORD PTR [edi+ecx-DWORD]
		punpcklbw MM1,MM7

Label2:	movd MM0,DWORD PTR [edi+ecx+DWORD]
		movd MM2,DWORD PTR [edx+ecx]
		movd MM3,DWORD PTR [esi+ecx]

		punpcklbw MM0,MM7
		punpcklbw MM2,MM7
		punpcklbw MM3,MM7

		paddw MM0,MM1
		paddw MM0,MM2
		paddw MM0,MM3

		psrlw MM0,2
		movq MM1,MM0
		packuswb MM0,MM7
		movd DWORD PTR [edi+ecx],MM0
		add ecx,4
		jnz Label2
		add edi,ebp
		inc ebx
		jnz Label1
		shr eax,2

		popad
	ret
ScreenFilter_Blur_Mmx ENDP


ALIGN 16
;
; permuter les composants rouges et bleus, avec prservation des alphas
; note : MM0  MM4 sont modifis
;
; Syntax :
; mov edi,{OFFSET de la structure Screen_Area de l'cran  altrer}
; call ScreenFilter_SwapRedsBlues_Mmx
;
; Return :
; rien
;
ScreenFilter_SwapRedsBlues_Mmx PROC
		push ecx

		mov ecx,(Screen_Area PTR [edi]).Screen_Size

		and ecx,11111111111111111111111111111000b
		jz Label2
		push edi
		mov edi,(Screen_Area PTR [edi]).Pointer
		add edi,ecx
		neg ecx

		movq MM1,QWORD PTR _Mem_Simd_Mask_Blues_
		movq MM2,QWORD PTR _Mem_Simd_Mask_Reds_
Label1:	movq MM0,QWORD PTR [edi+ecx]
		movq MM3,MM0
		movq MM4,MM0
		pand MM3,MM1
		pand MM4,MM2
		pxor MM0,MM3
		pxor MM0,MM4
		pslld MM3,16
		psrld MM4,16
		por MM0,MM3
		por MM0,MM4
		movq QWORD PTR [edi+ecx],MM0
		add ecx,QWORD
		jnz Label1
		pop edi

Label2:
		pop ecx
	ret
ScreenFilter_SwapRedsBlues_Mmx ENDP


;
; Strings
;
.DATA
ALIGN 16
_String_X_Screen_Pos_			DWORD ?
_String_Y_Screen_Pos_			DWORD ?
_String_Width_Loop_				DWORD ?
_String_Font_Pointer_			DWORD ?
_String_Font_Color_				DWORD 000FFFFFFh
_String_Back_Color_				DWORD 000000000h
_String_Digit_Pointer_			DWORD ?
_String_Digit_Width_x4_			DWORD ?
_String_Next_Digit_				DWORD ?
_String_Font_Height_			DWORD ?


.CODE
ALIGN 16
;
; convertir un dword en texte au format dcimale (routine la plus stable pour le travail)
;
; Syntax :
; mov eax,{la valeur  convertir en chane}
; mov esi,{OFFSET de la chaine de caractres  crer}
; call ConvertValToDec
;
; Return :
; eax = la longueur de la chane cre
;
ConvertValToDec PROC
		test eax,eax
		jz CasZero

		push ebx
		push ecx
		push edx
		push esi

		mov ecx,eax
		inc eax
		jz CasMoinsUn

		mov edx,2814749767
		mul edx
		shr edx,16
		jz Label01
		mov eax,100000
		mov ebx,edx
		mul edx
		sub ecx,eax
		mov edx,ebx

		mov eax,429497
		mov ebx,10
		mul edx
		jc Label02
		dec esi
		mul ebx
		jc Label03
		dec esi
		mul ebx
		jc Label04
		dec esi
		mul ebx
		jc Label05
		dec esi
		jmp Label06

Label01:	mov eax,429497
		mov ebx,10
		sub esi,5
		mul ecx
		jc Label07
		dec esi
		mul ebx
		jc Label08
		dec esi
		mul ebx
		jc Label09
		dec esi
		mul ebx
		jc Label10
		dec esi
		jmp Label11

Label02:	add dl,"0"
		mov BYTE PTR [esi],dl
		mul ebx
Label03:	add dl,"0"
		mov BYTE PTR [esi+1],dl
		mul ebx
Label04:	add dl,"0"
		mov BYTE PTR [esi+2],dl
		mul ebx
Label05:	add dl,"0"
		mov BYTE PTR [esi+3],dl
Label06:	mul ebx
		add dl,"0"
		mov BYTE PTR [esi+4],dl
		mov eax,429497
		mul ecx
Label07:	add dl,"0"
		mov BYTE PTR [esi+5],dl
		mul ebx
Label08:	add dl,"0"
		mov BYTE PTR [esi+6],dl
		mul ebx
Label09:	add dl,"0"
		mov BYTE PTR [esi+7],dl
		mul ebx
Label10:	add dl,"0"
		mov BYTE PTR [esi+8],dl
Label11:	mul ebx
		add dx,"0"
		mov WORD PTR [esi+9],dx
		lea eax,[esi+10]

		pop esi
		pop edx
		pop ecx
		pop ebx
		sub eax,esi
	ret

CasMoinsUn:
		mov DWORD PTR [esi],"4924"
		mov DWORD PTR [esi+4],"2769"
		mov WORD PTR [esi+8],"59"
		mov BYTE PTR [esi+10],0
		mov eax,10

		pop esi
		pop edx
		pop ecx
		pop ebx
	ret

CasZero:
		mov WORD PTR [esi],"0"
		inc eax
	ret
ConvertValToDec ENDP


ALIGN 16
;
; dfinir les couleurs (caractres + fond) d'une chane de caractres
; note : ca peut paraitre stupide de crer une procdure avec seulement 2 instructions, mais ca permet d'eviter
; de rpeter cette opration systmatiquement dans AfficherChaine (gain de vitesse), et c'est aussi quelque chose
; qui peut tre appel un nombre de fois considrable (donc, une macro prendrai beucoup plus d'espace mmoire)
;
; Syntax :
; mov eax,{couleur de la police de caractres (format A,R,V,B)}
; mov ebx,{couleur de fond de la police de caractres (format A,R,V,B)}
; call SetGfxStringColors
;
; Return :
; rien, mais les variables _String_Font_Color_ et _String_Back_Color_ qui servent pour AfficherChaine est dfinie
;
SetGfxStringColors PROC
		mov _String_Font_Color_,eax
		mov _String_Back_Color_,ebx
	ret
SetGfxStringColors ENDP


ALIGN 16
;
; obtenir la largeur et la hauteur (en pixels) d'une chane de caractres a afficher en fonction de la police
; utilise
;
; Syntax :
; mov edx,{adresse de la police de caratres  utiliser}
; mov esi,{adresse de la chaine  afficher}
; call GetGfxStringSizes
;
; Return :
; eax = la largeur de la chane  afficher (en pixels)
; ebx = la hauteur de la chane  afficher (en pixels)
;
GetGfxStringSizes PROC
		push ecx
		push edx
		push esi
		push edi

		xor ebx,ebx
		mov ecx,edx
		mov edi,DWORD PTR (Font_Header PTR [edx]).Font_Height
		shl edi,2
		add edi,DWORD

Label01:	movzx eax,BYTE PTR [esi]
		test eax,eax
		jz Label04

		cmp eax,32
		jb Label03
		cmp eax,DWORD PTR (Font_Header PTR [ecx]).Number_Of_Digit
		jbe Label02
		mov eax,127
Label02:	sub eax,32

		mul edi
		add eax,ecx

		add ebx,DWORD PTR [eax+8]
Label03:	inc esi
		jmp Label01
Label04:	mov eax,ebx
		mov ebx,DWORD PTR [ecx]

		pop edi
		pop esi
		pop edx
		pop ecx
	ret
GetGfxStringSizes ENDP


ALIGN 16
;
; Afficher une chane de caractres centre (largeur maximale des caractres de la police = 32 pixels, (ce qui
; est suffisant dans la plupart des cas...))
; (cette version considre que tous les alphas de la couleur de la police ont la mme valeur, ce qui est
; gnralement le cas...)
;
; Syntax :
; mov eax,{position X (largeur en pixels) de dpart}
; mov ebx,{position Y (hauteur en pixels) de dpart}
; mov edx,{adresse de la police de caratres  utiliser}
; mov esi,{adresse de la chaine  afficher}
; mov edi,{OFFSET de la structure cran}
; call DisplayCenteredGfxString_Mmx
;
; Return :
; rien, mais la chaine est affiche  l'cran
;
DisplayCenteredGfxString_Mmx PROC
		push eax
		push ebx
		push ecx
		push ebx

		mov ecx,eax
		call GetGfxStringSizes

		shr eax,1
		sub ecx,eax
		mov eax,ecx
		mov ecx,(Font_Header PTR [edx]).Font_Height
		shr ecx,1
		sub ebx,ecx
		mov ecx,ebx
		pop ebx
		sub ebx,ecx
		call DisplayGfxString_Mmx

		pop ecx
		pop ebx
		pop eax
	ret
DisplayCenteredGfxString_Mmx ENDP


ALIGN 16
;
; Afficher une chane de caractres (largeur maximale des caractres de la police = 32 pixels, (ce qui est suffisant
; dans la plupart des cas...))
;
; Syntax :
; mov eax,{position X (largeur en pixels) de dpart}
; mov ebx,{position Y (hauteur en pixels) de dpart}
; mov edx,{OFFSET de la police de caratres  utiliser}
; mov esi,{OFFSET de la chaine  afficher}
; mov edi,{OFFSET de la structure cran}
; call DisplayGfxString_Mmx
;
; Return :
; rien, mais la chaine est affiche  l'cran (attention, tous les registre MMx sont modifis)
;
DisplayGfxString_Mmx PROC
		pushad

		mov ebp,edi
		mov _String_Font_Pointer_,edx
		mov _String_Digit_Pointer_,esi
		mov esi,(Font_Header PTR [edx]).Font_Height
		mov _String_Font_Height_,esi

		shl eax,2
		mov _String_X_Screen_Pos_,eax

		mov ecx,(Screen_Area PTR [ebp]).Screen_Width_x4
		mov edi,(Screen_Area PTR [ebp]).Pointer

		mov eax,ecx
		imul ebx

		add edi,eax

		mov eax,ecx
		mul _String_Font_Height_
		mov _String_Next_Digit_,eax
		mov ebx,_String_X_Screen_Pos_
		movq MM5,QWORD PTR _Mem_Simd_Mask_RVB_unpck_1_
		movq MM6,QWORD PTR _Mem_Simd_Mask_RVB_unpck_256_
		pxor MM7,MM7

Label01:	mov edx,_String_Digit_Pointer_
		movzx eax,BYTE PTR [edx]
		test eax,eax
		jz Label14

		cmp eax,32
		jb Label07
		mov esi,_String_Font_Pointer_
		mov ecx,(Font_Header PTR [esi]).Font_Height
		shl ecx,2
		cmp eax,(Font_Header PTR [esi]).Number_Of_Digit
		jbe Label02
		mov eax,127
Label02:	sub eax,32

		add ecx,4
		mul ecx
		add esi,eax

		mov eax,DWORD PTR [esi+8]
		shl eax,2
		mov _String_Digit_Width_x4_,eax

		mov eax,_String_Font_Height_
		shl eax,2
		add esi,eax
		neg eax

		mov edx,(Screen_Area PTR [ebp]).Screen_Size
		mov ecx,edi
		sub ecx,(Screen_Area PTR [ebp]).Pointer
		cmp ecx,edx
		jae Label08
		add ecx,_String_Next_Digit_
		cmp ecx,edx
		jae Label08
		mov edx,(Screen_Area PTR [ebp]).Screen_Width_x4
		mov ecx,ebx
		cmp ecx,edx
		jae Label08
		add ecx,_String_Digit_Width_x4_
		cmp ecx,edx
		jae Label08

Label03:	mov ecx,DWORD PTR [esi+eax+12]
		mov edx,_String_Digit_Width_x4_
		mov _String_Width_Loop_,edx

Label04:	add ecx,ecx
		jnc Label05
		mov edx,_String_Font_Color_
		jmp Label06
Label05:	mov edx,_String_Back_Color_

Label06:	movd MM0,edx
		movd MM1,DWORD PTR [edi+ebx]
		punpcklbw MM0,MM7
		punpcklbw MM1,MM7
		movq MM2,MM0
		pshufw MM2,MM2,0FFh
		movq MM3,MM6
		psubw MM3,MM2
		paddw MM2,MM5
		pmullw MM1,MM3
		pmullw MM0,MM2
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM7
		movd DWORD PTR [edi+ebx],MM0
		add ebx,4

		sub _String_Width_Loop_,4
		jnz Label04

		sub ebx,_String_Digit_Width_x4_
		add edi,(Screen_Area PTR [ebp]).Screen_Width_x4

		add eax,4
		jnz Label03

		sub edi,_String_Next_Digit_
		add ebx,_String_Digit_Width_x4_

Label07:	inc _String_Digit_Pointer_
		jmp Label01

Label08:	mov ecx,edi
		sub ecx,(Screen_Area PTR [ebp]).Pointer
		cmp ecx,(Screen_Area PTR [ebp]).Screen_Size
		jae Label13

		mov ecx,DWORD PTR [esi+eax+12]
		mov edx,_String_Digit_Width_x4_
		mov _String_Width_Loop_,edx

Label09:	add ecx,ecx
		jnc Label10
		cmp ebx,(Screen_Area PTR [ebp]).Screen_Width_x4
		jae Label12
		mov edx,_String_Font_Color_
		jmp Label11
Label10:	cmp ebx,(Screen_Area PTR [ebp]).Screen_Width_x4
		jae Label12
		mov edx,_String_Back_Color_

Label11:	movd MM0,edx
		movd MM1,DWORD PTR [edi+ebx]
		punpcklbw MM0,MM7
		punpcklbw MM1,MM7
		movq MM2,MM0
		pshufw MM2,MM2,0FFh
		movq MM3,MM6
		psubw MM3,MM2
		paddw MM2,MM5
		pmullw MM1,MM3
		pmullw MM0,MM2
		paddw MM0,MM1
		psrlw MM0,8
		packuswb MM0,MM7
		movd DWORD PTR [edi+ebx],MM0
Label12:	add ebx,4

		sub _String_Width_Loop_,4
		jnz Label09

		sub ebx,_String_Digit_Width_x4_
Label13:	add edi,(Screen_Area PTR [ebp]).Screen_Width_x4

		add eax,4
		jnz Label08

		sub edi,_String_Next_Digit_
		add ebx,_String_Digit_Width_x4_
		inc _String_Digit_Pointer_
		jmp Label01
Label14:
		popad
	ret
DisplayGfxString_Mmx ENDP


